'use strict';

const common = require('../common');
const DBErrors = require('./errors');

const _fileScope = common.fileScope(__filename);

class BaseDatabase {
  constructor(logger) {
    this.logger = logger;
  }

  static _camelfy(snakeCase, delim = '_') {
    if (!snakeCase || typeof snakeCase.split !== 'function') {
      return undefined;
    }
    const words = snakeCase.split(delim);
    return [
      words.shift(),
      ...words.map((word) => word.charAt(0).toUpperCase() + word.slice(1)),
    ].join('');
  }

  _notImplemented(method, args) {
    const _scope = _fileScope(method);
    this.logger.error(_scope, 'abstract method called', Array.from(args));
    throw new DBErrors.NotImplemented();
  }

  async context(fn) {
    this._notImplemented('context', { fn });
  }

  async transaction(dbCtx, fn) {
    this._notImplemented('transaction', { dbCtx, fn });
  }

  async getAuthById(dbCtx, id) {
    this._notImplemented('getAuthById', { dbCtx, id });
  }

  async insertLink(dbCtx, id, url, authToken) {
    this._notImplemented('insertLink', { dbCtx, id, url, authToken });
  }

  async getLinkById(dbCtx, id) {
    this._notImplemented('getLinkById', { dbCtx, id });
  }

  async getLinkByUrl(dbCtx, url) {
    this._notImplemented('getLinkByUrl', { dbCtx, url });
  }

  async accessLink(dbCtx, id) {
    this._notImplemented('accessLink', { dbCtx, id });
  }

  async expireLink(dbCtx, id, expires) {
    this._notImplemented('expireLink', { dbCtx, id, expires });
  }

  async updateLink(dbCtx, id, url) {
    this._notImplemented('updateLink', { dbCtx, id, url });
  }

  async getAllLinks(dbCtx) {
    this._notImplemented('getAllLinks', { dbCtx });
  }

}

module.exports = BaseDatabase;
