'use strict';

const common = require('../common');
const DBErrors = require('./errors');

const _fileScope = common.fileScope(__filename);

const defaultOptions = {
  connectionString: '',
};

class DatabaseFactory {
  constructor(logger, options, ...rest) {
    const _scope = _fileScope + ':constructor';
    options = Object.assign({}, defaultOptions, options);
    const protocol = options.connectionString.slice(0, options.connectionString.indexOf('://')).toLowerCase();
    // eslint-disable-next-line sonarjs/no-small-switch
    switch (protocol) {
      case DatabaseFactory.Engines.PostgreSQL: {
        const Postgres = require('./postgres');
        return new Postgres(logger, options, ...rest);
      }

      case DatabaseFactory.Engines.SQLite: {
        const SQLite = require('./sqlite');
        return new SQLite(logger, options, ...rest);
      }

      default:
        logger.error(_scope, 'unsupported connectionString', { options });
        throw new DBErrors.UnsupportedEngine(protocol);
    }
  }

  static get Engines() {
    return {
      PostgreSQL: 'postgresql',
      SQLite: 'sqlite',
    };
  }

}

module.exports = DatabaseFactory;
