'use strict';

const common = require('./common');
const { createHash } = require('crypto');

/**
 * TODO: base62 would be slightly prettier
 * TODO: filter potential horrors
 * @param {string} seed
 * @param {number} maxLength
 */
async function newSlug(seed, maxLength = 86) {
  let slug;
  if (maxLength > 86) {
    maxLength = 86;
  }
  if (typeof seed !== 'undefined') {
    const hash = createHash('sha512');
    hash.update(seed);
    slug = hash.digest('base64');
  } else {
    const slugRaw = await common.randomBytesAsync(Math.round(maxLength * 3 / 4));
    slug = slugRaw.toString('base64');
  }
  return slug.slice(0, maxLength).replace('/', '-').replace('+', '.');
}


async function* makeSlugGenerator(seed, initialLength = 5, maxLength = 22) {
  let length = initialLength;
  let slug = await newSlug(seed, maxLength);
  while (true) {
    yield slug.slice(0, length);
    length++;
    if (length > maxLength) {
      length = initialLength;
      slug = await newSlug(undefined, maxLength); // Hash not suitable, try randomness.
    }
  }
}

module.exports = {
  makeSlugGenerator,
  newSlug,
};