/* eslint-env mocha */
/* eslint-disable capitalized-comments */

'use strict';

const assert = require('assert');
const sinon = require('sinon');
const BaseDatabase = require('../../../src/db/base');
const DBErrors = require('../../../src/db/errors');

const noExpectedException = 'did not get expected exception';

describe('BaseDatabase', function () {
  let logger, db;

  beforeEach(function () {
    logger = { error: () => {} };
    // logger = console;
    db = new BaseDatabase(logger);
  });

  afterEach(function () {
    sinon.restore();
  });

  describe('_camelfy', function () {
    it('empty arg', function () {
      const result = BaseDatabase._camelfy();
      assert.strictEqual(result, undefined);
    });
    it('no change', function () {
      const str = 'camelCase';
      const result = BaseDatabase._camelfy(str);
      assert.strictEqual(result, str);
    });
    it('does expected', function () {
      const str = 'snake_case_thing';
      const result = BaseDatabase._camelfy(str);
      assert.strictEqual(result, 'snakeCaseThing');
    });
  }); // _camelfy

  describe('interface', function () {
    it('covers methods', async function () {
      const methods = [
        'context',
        'transaction',
        'getAuthById',
        'insertLink',
        'getLinkById',
        'getLinkByUrl',
        'accessLink',
        'expireLink',
        'updateLink',
        'getAllLinks',
      ];
      const invokedMethods = methods.map(async (m) => {
        try {
          // eslint-disable-next-line security/detect-object-injection
          await db[m]();
          assert.fail(noExpectedException);
        } catch (e) {
          assert(e instanceof DBErrors.NotImplemented);
        }
      });
      await Promise.all(invokedMethods);
    });
  }); // interface

});