/* eslint-env mocha */
/* eslint-disable capitalized-comments */

'use strict';

const assert = require('assert');
// eslint-disable-next-line node/no-unpublished-require
const sinon = require('sinon');
const DatabaseFactory = require('../../../src/db');
const DBErrors = require('../../../src/db/errors');

const noExpectedException = 'did not get expected exception';

describe('DatabaseFactory', function () {
  let db, logger, options;

  beforeEach(function () {
    logger = {
      debug: () => {},
      error: () => {},
      info: () => {},
    };
    options = {};
  });

  afterEach(function () {
    sinon.restore();
  });

  it('fails on invalid engine', function () {
    try {
      new DatabaseFactory(logger, options);
      assert.fail(noExpectedException);
    } catch (e) {
      assert(e instanceof DBErrors.UnsupportedEngine);
    }
  });

  it('creates sqlite engine', function () {
    this.slow(500);
    options.connectionString = 'sqlite://';
    db = new DatabaseFactory(logger, options);
    assert.strictEqual(db.constructor.name, 'SQLiteDatabase');
  });

  it('creates postgres engine', function () {
    this.slow(500);
    const stubPgp = sinon.stub();
    stubPgp.utils = {
      enumSql: sinon.stub().returns({}),
    };
    stubPgp.QueryFile = sinon.stub().returns({});
    options.connectionString = 'postgresql://';
    db = new DatabaseFactory(logger, options, stubPgp);
    assert.strictEqual(db.constructor.name, 'PostgresDatabase');
  });

});