/* eslint-env mocha */
/* eslint-disable capitalized-comments */

'use strict';

const assert = require('assert');
const sinon = require('sinon');

describe('slug', function () {
  const Slug = require('../../src/slug');

  const maxSlugLength = 86;

  afterEach(function () {
    sinon.restore();
  });

  describe('newSlug', function () {
    it('generates a slug from seed', async function () {
      const seed = '';
      const seedSHA512b64trunc = 'z4PhNX7vuL3xVChQ1m2AB9Yg5AULVxXcg-SpIdNs6c5H0NE8XYXysP.DGNKHfuwvY7kxvUdBeoGlODJ6+SfaPg';
      const result = await Slug.newSlug(seed);
      assert.strictEqual(result, seedSHA512b64trunc);
    });
    it('generates a random slug', async function () {
      const result = await Slug.newSlug();
      assert.strictEqual(result.length, maxSlugLength);
    });
    it('only makes a slug so big', async function () {
      const slugLen = 222;
      const result = await Slug.newSlug(undefined, slugLen);
      assert.strictEqual(result.length, maxSlugLength);
    });
  }); // newSlug

  describe('makeSlugGenerator', function () {
    it('makes a slug generator which generates a slug', async function () {
      const seed = '';
      const sg = Slug.makeSlugGenerator(seed);
      const { done, value } = await sg.next();
      assert(!done);
      assert.strictEqual(value.length, 5);
    });
    it('makes a slug generator which generates many slugs', async function () {
      const seed = '';
      const sg = Slug.makeSlugGenerator(seed, 5, 8);

      const checkNext = async (expected) => {
        const { done, value } = await sg.next();
        assert(!done);
        assert.strictEqual(value, expected);
      };
      const checkNextLen = async (expected) => {
        const { done, value } = await sg.next();
        assert(!done);
        assert.strictEqual(value.length, expected);
      };

      await checkNext('z4PhN');
      await checkNext('z4PhNX');
      await checkNext('z4PhNX7');
      await checkNext('z4PhNX7v');
      await checkNextLen(5);
      await checkNextLen(6);
      await checkNextLen(7);
      await checkNextLen(8);
      await checkNextLen(5);
      await checkNextLen(6);
    });
  }); // makeSlugGenerator

}); // slug